
from casatools import image, quanta
from matplotlib import pyplot as pl
from matplotlib.patches import Ellipse
import numpy as np
import subprocess

from astropy.wcs import WCS


def plot_image(imname: str, imtype="", chan=0, trim=False, outsize="", pixrange=[]):
    """
        Utility function to produce a .png plot from a CASA image file.

    Keyword arguments:
        imname (str, required):     CASA image file name
        imtype (str, optional):     Additional image type qualifiers. Defaults to "". Example: moment8, residual
        chan (int, optional):       Channel number. Defaults to 0.
        trim (bool, optional):      Apply imagemagick mogrify -trim to plot. Defaults to False.
        outsize(str, optional):     Apply imagemagick mogrify -resize to plot. Defaults to ""(no resizing). Ex. "256" ,  "150%" 
        pixrange (list, optional):  Set a range of pixel intesities to be plotted [min, max]
    """

    ia = image()
    qa = quanta()

    ia.open(imname + imtype)
    pix = ia.getchunk()[:, :, 0, chan]
    csys = ia.coordsys()
    bminfo = ia.restoringbeam()
    beam = None
    if len(bminfo):
        beam = ia.commonbeam() 
    ia.close()
    shp = pix.shape

    rad_to_deg = 180 / np.pi
    w = WCS(naxis=2)
    w.wcs.crpix = csys.referencepixel()['numeric'][0:2]
    w.wcs.cdelt = csys.increment()['numeric'][0:2] * rad_to_deg
    w.wcs.crval = csys.referencevalue()['numeric'][0:2] * rad_to_deg
    w.wcs.ctype = ['RA---SIN', 'DEC--SIN']

    ax = pl.subplot(projection=w)

    #p1 = int(shp[0] * 0.25)
    #p2 = int(shp[0] * 0.75)
    p1 = int(shp[0]) - 1
    p2 = int(shp[1]) - 1
    # Plot beam if exists
    if beam:
        x = p1*0.1
        y = p2*0.1
        bmajd = qa.convert(beam['major'],'deg')['value']
        bmind = qa.convert(beam['minor'],'deg')['value'] 
        bpa = beam['pa']['value']
        pixscale = w.wcs.cdelt[1]
        el = Ellipse((x,y), width=bmind/pixscale, height=bmajd/pixscale, angle=bpa, fill=False, edgecolor="white")
        ax.add_artist(el)

    if pixrange != [] and len(pixrange) == 2:
        pmin = pixrange[0]
        pmax = pixrange[1]
    else:
        pmin=None
        pmax=None
    #pl.imshow(pix[p1:p2, p1:p2].transpose(), origin='lower', cmap=pl.cm.viridis)
    pl.imshow(pix[0:p1, 0:p2].transpose(), origin='lower', cmap=pl.cm.viridis, vmin=pmin, vmax=pmax)
    pl.xlabel('Right Ascension')
    pl.ylabel('Declination')

    #pl.imsave(imname + imtype + '.png', pix[p1:p2, p1:p2].transpose())
    #pl.imsave(imname + imtype + '.png', pix[0:p1, 0:p2].transpose(), vmin=pmin, vmax=pmax)
    # use savefig 
    pl.savefig(imname + imtype + '.png')

    if trim is True:
    #if trim is True or outsize is not "":
    #    cmd = 'mogrify '
    #    if trim:
    #        cmd += '-trim '

    #    trimmedoutsize = outsize.strip()
    #    if len(trimmedoutsize):
    #        if trimmedoutsize.isnumeric() or trimmedoutsize.find('x') or trimmedoutsize.endswith('%'):
    #           cmd += ' -resize ' + trimmedoutsize + ' '
       
        subprocess.call('mogrify -trim ' + imname + imtype + '.png', shell=True)
    #    subprocess.call(cmd + imname + imtype + '.png', shell=True)
